/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.simpleds.annotations.processor;

import javax.inject.Singleton;

import org.loom.annotation.processor.AbstractPropertyAnnotationProcessor;
import org.loom.mapping.Event;
import org.loom.validator.AbstractRequiredValidator;
import org.loom.validator.RequiredValidatorFactory;
import org.simpleds.annotations.Property;


/**
 * Processes {@link Property} annotations
 * @author icoloma
 *
 */
@Singleton
public class PropertyAnnotationProcessor extends AbstractPropertyAnnotationProcessor<Property, Property> {

	public PropertyAnnotationProcessor() {
		super(Property.class, null);
	}
	
	@Override
	protected void process(Event event, Property annotation, String propertyPath) {
		if (annotation.required()) {
			Class<?> propertyClass = event.getPropertyClass(propertyPath);
			AbstractRequiredValidator<?> v = RequiredValidatorFactory.createValidator(propertyClass);
			v.setPropertyPath(propertyPath);
			event.addValidator(v);
		}
	}

}
