package org.loom.simpleds.json;

import java.util.List;

import org.codehaus.jackson.annotate.JsonAutoDetect;
import org.codehaus.jackson.annotate.JsonMethod;
import org.codehaus.jackson.annotate.JsonProperty;
import org.codehaus.jackson.map.annotate.JsonDeserialize;
import org.simpleds.CursorList;

import com.google.appengine.api.datastore.Cursor;

/**
 * Mixin used to serialize {@link CursorList} instances using Jackson.
 * To use, invoke
 * <pre>
 * objectMapper.getSerializationConfig().addMixInAnnotations(CursorList.class, CursorListJacksonMixIn.class);
 * </pre>
 * Details about mixins in Jackson can be seen <a href="http://wiki.fasterxml.com/JacksonMixInAnnotations">here</a>
 * @author icoloma
 *
 */
@JsonAutoDetect(JsonMethod.NONE)
@JsonDeserialize(as=CursorList.class)
public interface CursorListJacksonMixin {

	  @JsonProperty 
	  List<Object> getData();
	  
	  @JsonProperty
	  Cursor getCursor();
	  
}
